#include "stm32f10x_lib.h"

//function prototypes:
void RCC_config(void);
void GPIO_config(void);
void NVIC_config(void);

////////////////////////////////main///////////////////////////////////////////
int main()
{
  RCC_config();
  NVIC_config();
  GPIO_config();
  
  USART_InitTypeDef USART_InitStructure;
  USART_InitStructure.USART_BaudRate = 9600; //Baudrate
  USART_InitStructure.USART_WordLength = USART_WordLength_8b; //8 Databits
  USART_InitStructure.USART_StopBits = USART_StopBits_1; //1 Stopbit
  USART_InitStructure.USART_Parity = USART_Parity_No ; //No Parity
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None; //Hardware Flow Control off
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx; //enable TX and RX

  USART_Init(USART2, &USART_InitStructure);
  
  USART_ITConfig(USART2, USART_IT_TXE, ENABLE); //TX empty Interrupt
  USART_ITConfig(USART2, USART_IT_RXNE, ENABLE); //RX not empty Interrupt
  
  USART_Cmd(USART2, ENABLE); //enable USART2
  
  while(1);
}

///////////////////////////clock-config///////////////////////////////////////
void RCC_config(void)
{
  ErrorStatus HSEStartUpStatus;
  RCC_DeInit(); //reset

  RCC_HSEConfig(RCC_HSE_ON); //turn on external oscillator

  HSEStartUpStatus = RCC_WaitForHSEStartUp(); //check if HSE ready, timeout if not ready

  if(HSEStartUpStatus == SUCCESS) //if startup was correct
  {
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable); // Enable Prefetch Buffer (when AHB != 1, enable to be sure...)
    FLASH_SetLatency(FLASH_Latency_2); //2 wait state, because clock between 48Mhz and 72Mhz
    RCC_HCLKConfig(RCC_SYSCLK_Div1); //HCLK(AHB)-Clock = SYSCLK (I think peripheral clock)
    RCC_PCLK2Config(RCC_HCLK_Div1); //PCLK2 = HCLK (for APB1 devices, high speed APB)
    RCC_PCLK1Config(RCC_HCLK_Div2); //PCLK1 = HCLK/2 (for APB2 devices, low speed APB)
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9); //PLL Source HSE_Clock, Multiplier 9x (72Mhz)
    RCC_PLLCmd(ENABLE); //enable PLL

    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET); //wait until PLL is ready

    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK); //PLL as System Clock Source
    
    while(RCC_GetSYSCLKSource() != 0x08); //wait until PLL is used as system clock source
  }
    
  /* Enable GPIOD and AFIO (Alternate Function I/O) clocks (APB2, HIGH SPEED)*/
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);

  /* Enable USART2 clocks (APB1, LOW SPEED)*/
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
}


///////////////////////////////////GPIO config/////////////////////////////////
void GPIO_config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure; //structure 

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2; //RTS and TX of USART2
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz; //50Mhz Speed
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; //Push-Pull-Out
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_3; //CTS and RX of USART2
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING; //input floating
  GPIO_Init(GPIOA, &GPIO_InitStructure);
}

////////////////////////////////////NVIC config/////////////////////////////////
void NVIC_config(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;
  
  NVIC_DeInit(); //debugging...
  NVIC_SCBDeInit(); //debugging...
  
  //set Vector-Tabel to Flash at 0x08000000
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
 
  //enable USART2 Interrupt
  NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}
